<?php
if( !defined('ABSPATH') )
	exit();
if( !class_exists('PT_API_Endpoint') ){
	class PT_API_Endpoint{
		var $post_type	=	'wp-ticket';
		var $post_status = 'publish';
		var $department = 'department';
		var $posts_per_page = 20;
		var $offset = 0;
		### Order ticket list 
		var $orderby = 'ID';
		var $order = 'DESC';
		var $comment_type = 'reply_ticket';
		var $avatar = 64;
		var $endpoint_slug = 'pressticket';
		public function __construct(){
			add_filter('query_vars', array($this, 'add_query_vars'), 0);
			add_action('parse_request', array($this, 'sniff_requests'), 0);
			add_action('init', array($this, 'add_endpoint'), 0);
		}	
		
		/** Add public query vars
		*	@param array $vars List of current public query vars
		*	@return array $vars 
		*/
		public function add_query_vars($vars){
			$vars[] = '__api';
			$vars[] = 'action';
			### OpenTicket Parameter.
			$vars[] = 'ticketid';
			$vars[] = 'username';
			$vars[] = 'apikey';
			$vars[] = 'department';
			$vars[] = 'priority';
			$vars[] = 'status';
			$vars[] = 'subject';
			$vars[] = 'content';
			$vars[] = 'response_content';
			$vars[] = 'offset';
			return $vars;
		}
		
		/** Add API Endpoint
		*	@return void
		*/
		public function add_endpoint(){
			$slug = wpo_ticket_option_endpoint_slug() ? wpo_ticket_option_endpoint_slug() : $this->endpoint_slug;
			add_rewrite_rule('^api/'.$slug.'/?(.*)?/?','index.php?__api=pressticket&action=$matches[1]','top');
		}
		/**	Sniff Requests
		*	This is where we hijack all API requests
		* 	If $_GET['__api'] is set, we kill WP and serve up pug bomb awesomeness
		*	@return die if API request
		*/
		public function sniff_requests(){
			global $wp;
			if(isset($wp->query_vars['__api'])){
				$this->handle_request();
				exit;
			}
		}
		function get_userid($username){
			global $wpdb;
			if( !$username )
				return 1;
			return $query = $wpdb->get_var("SELECT `ID` FROM $wpdb->users WHERE `user_login`='$username'");
		}
		/**
		 * Verify API key
		 * @param string $username
		 * @param string $apikey
		 */
		function verify_apikey($username, $apikey){
			global $wpdb;
			if( !$username || !$apikey )
				return false;
			$query = $wpdb->get_var("SELECT `user_id` FROM $wpdb->usermeta WHERE `meta_value`='$apikey'");
			if( !$query ){
				return FALSE;
			}
			else{
				$user_id = (int)$query;
				$query_userdata = $wpdb->get_var("SELECT `user_login` FROM $wpdb->users WHERE `ID`=$user_id");
				if( $query_userdata && $query_userdata == $username )
					return true;
				return false;	
			}
		}
		function action($action){
			global $wp;
			$username = $wp->query_vars['username'];
			$user_id = $this->get_userid( trim( $username ) );
			$apikey = $wp->query_vars['apikey'];
			### 1, check Helpdesk Status
			if( wpo_ticket_option_helpdesk_status()=='no' )
				return $this->send_response('error',wpo_ticket_option_helpdesk_status_messages());
			### 2, check API
			if( $this->verify_apikey($username, $apikey) !== true )
				return $this->send_response('error',__('Invalid Username or APIKEY!','wpo'));
			### 3, check permission
			if(  !user_can($user_id, wpo_ticket_cap_view_ticket())  )
				return $this->send_response('error',__('You do not have the right permission to use Our Ticket System!','wpo'));
			
			switch ($action) {
				case 'openticket':
					return $this->openticket();
				break;
				case 'getposts':
					return $this->getposts();
				break;
				case 'getpost':
					return $this->getpost();
				break;
				case 'getresponses':
					return $this->getresponses();
				break;
				case 'sendresponse':
					return $this->sendresponse();
				break;
				case 'verifymy':
					return $this->verifymy();
				break;
				default:
					$this->send_response( 'error', __('Action isn\'t available!','wpo') );
				break;
			}
		}
		function verifymy(){
			return;
		}
		function openticket(){
			global $wp;
			### post
			$post_title = trim( $wp->query_vars['subject'] ) ? trim( $wp->query_vars['subject'] ) : null;
			$post_content = trim( $wp->query_vars['content'] ) ? trim( $wp->query_vars['content'] ) : null;
			$username = trim( $wp->query_vars['username'] ) ? trim( $wp->query_vars['username'] ) : null;
			### taxonomy
			$department = trim( $wp->query_vars['department'] ) ? trim( $wp->query_vars['department'] ) : null;
			### meta
			$priority = trim( $wp->query_vars['priority'] ) ? trim( $wp->query_vars['priority'] ) : 'normal';
			$status = trim( $wp->query_vars['status '] ) ? trim( $wp->query_vars['status '] ) : 'open';
			
			if( !$department )
				$this->send_response('error',__('Please choose a Department.','wpo'));
			
			$postdata = array(
				'post_title'	=>	$post_title,
				'post_content'	=>	$post_content,
				'post_author'	=>	$this->get_userid($username),
				'post_type'	=>	$this->post_type,
				'post_status'	=>	$this->post_status,
				'comment_status'	=>	'closed'
			);
			$post_id = wp_insert_post($postdata);
			if( $post_id ){
				### set relate taxonomy, its department.
				//if( term_exists($department,$this->department) )
				wp_set_post_terms($post_id,$department,$this->department,true);
				### update meta
				update_post_meta($post_id, 'wpo_ticket_priority', $priority);
				update_post_meta($post_id, 'wpo_ticket_status', $status);
				update_post_meta($post_id, 'wpo_ticket_type', 'wp');	
				do_action('pressticket_add_ticket', $post_id,$department);	
				return $this->send_response('success',__('Successfully opened a Ticket!','wpo'), 
					array(
						'ticket_id'=>wpo_ticket_get_ticketID($post_id),
						'ticket_url'=>get_permalink($post_id)
					)
				);
			}
		}
		function getposts(){
			global $wp;
			$postdata = array();
			$user_id = $this->get_userid( trim( $wp->query_vars['username'] ) );
			$s = isset( $wp->query_vars['s']  ) ? trim( $wp->query_vars['s'] ) : NULL;
			$offset = isset( $wp->query_vars['offset']  ) ? trim( $wp->query_vars['offset'] ) : 0;
			$status = isset( $wp->query_vars['status']  ) ? trim( $wp->query_vars['status'] ) : NULL;
			$priority = isset( $wp->query_vars['priority']  ) ? trim( $wp->query_vars['priority'] ) : NULL;
			$args = array(
				'author'	=>	$user_id,
				'post_type'	=>	$this->post_type,
				'post_status'	=>	$this->post_status,
				'posts_per_page'	=>	$this->posts_per_page,
				'orderby'          => $this->orderby,
				'order'	=>	$this->order,
				'offset'	=>	$offset ? $offset : $this->offset
			);
			if( $s ){
				$args['meta_query'] = array(
					array(
						'key'	=>	'wpo_ticket_ID',
						'value'	=>	$s
					)
				);
			}
			if( $status ){
				$args['meta_query'] = array(
					array(
						'key'	=>	'wpo_ticket_status',
						'value'	=>	$status
					)
				);				
			}
			if( $priority ){
				$args['meta_query'] = array(
					array(
						'key'	=>	'wpo_ticket_priority',
						'value'	=>	$priority
					)
				);				
			}			
			$posts = get_posts($args);
			if( !empty( $posts ) ){
				foreach ( $posts as $ticket ){
					$postdata[] = array(
						'ticket_id'	=> wpo_ticket_get_ticketID( $ticket->ID ),
						'ticket_title'	=> $ticket->post_title,
						'ticket_content'	=>	$ticket->post_content,
						'ticket_status'	=>	wpo_ticket_get_ticket_status( $ticket->ID, false ),
						'ticket_status_key'	=>	wpo_ticket_get_ticket_status( $ticket->ID),
						'ticket_priority'	=>	wpo_ticket_get_ticket_priority( $ticket->ID , false ),
						'ticket_priority_key'	=>	wpo_ticket_get_ticket_priority( $ticket->ID),
						'ticket_department'	=>	wpo_pressticket_get_department($ticket->ID, true),
						'ticket_helptopic'	=>	get_post_meta($ticket->ID,'wpo_ticket_helptopics',true),
						'ticket_date'	=>	$ticket->post_date,
						'ticket_url'	=>	get_permalink( $ticket->ID ),
						'ticket_response_number'	=>	wpo_ticket_count_reply($ticket->ID),
						'ticket_orderby'	=> $this->orderby,
						'ticket_order'	=>	$this->order,
						'offset'	=>	$offset + $this->posts_per_page,
						'posts_per_page'	=>	$this->posts_per_page
					);
				}
				return $this->send_response('success', __('Get the List successfully','wpo'), $postdata );	
			}
			return $this->send_response('success', __('Get the List successfully','wpo') );
		}
		function getpost(){
			global $wp;
			$ticketid =  isset( $wp->query_vars['ticketid']  ) ? trim( $wp->query_vars['ticketid'] ) : NULL;
			$postdata = array();
			$post_id = wpo_ticket_get_postid_from_ticketid($ticketid);
			$post = get_post($post_id);
			if( $post ){
				$postdata = array(
					'ticket_id'	=>	$ticketid,
					'ticket_title'	=>	$post->post_title,
					'ticket_content'	=>	$post->post_content,
					'ticket_status'	=>	wpo_ticket_get_ticket_status( $post->ID, false ),
					'ticket_status_key'	=>	wpo_ticket_get_ticket_status( $post->ID ),
					'ticket_priority'	=>	wpo_ticket_get_ticket_priority( $post->ID , false ),
					'ticket_department'	=>	wpo_pressticket_get_department($post->ID, true),
					'ticket_helptopic'	=>	get_post_meta($ticket->ID,'wpo_ticket_helptopics',true),
					'ticket_date'	=>	$post->post_date,
					'ticket_url'	=>	get_permalink( $post->ID ),
					'ticket_response_number'	=>	wpo_ticket_count_reply($post->ID)					
				);
			}
			return $this->send_response('success', __('Get the Ticket successfully','wpo'), $postdata );
		}
		function getresponses(){
			global $wp;
			$ticketid =  isset( $wp->query_vars['ticketid']  ) ? trim( $wp->query_vars['ticketid'] ) : NULL;
			$postdata = array();
			$post_id = wpo_ticket_get_postid_from_ticketid($ticketid);
			$args = array(
				'post_id'	=>	$post_id,
				'type'	=>	$this->comment_type,
				'order' => 'DESC',
			);
			$responses = wpo_ticket_get_reply($post_id);
			if( $responses ){
				foreach ( $responses as $response ){
					$postdata[] = array(
						'response_author'	=>	$response['comment_author'],
						'response_content'	=>	$response['comment_content'],
						'response_date'	=>	$response['comment_date'],
						'response_avatar'	=>	get_avatar( $response['comment_author_email'], $this->avatar )
					);
				}
				return $this->send_response('success', __('Get the Responses successfully','wpo'), $postdata );
			}
			return $this->send_response('success', __('Get the Responses successfully','wpo'), NULL );
		}
		function sendresponse(){
			global $wp;
			$ticketid =  isset( $wp->query_vars['ticketid']  ) ? trim( $wp->query_vars['ticketid'] ) : NULL;
			$comment_author =  isset( $wp->query_vars['username']  ) ? trim( $wp->query_vars['username'] ) : NULL;
			$post_id = wpo_ticket_get_postid_from_ticketid($ticketid);
			$users = get_user_by('login', $comment_author);
			$response_content = isset( $wp->query_vars['response_content']  ) ? trim( $wp->query_vars['response_content'] ) : NULL;
			$status = isset( $wp->query_vars['status']  ) ? trim( $wp->query_vars['status'] ) : 'open';
			$data = array(
				'comment_post_ID'	=>	$post_id,
				'comment_author'	=>	$comment_author,
				'comment_author_email'	=>	$users->user_email,
				'comment_content'	=>	$response_content,
				//'comment_content'	=>	base64_decode( $response_content ),
				'comment_type'	=>	$this->comment_type,
				'user_id'	=>	$users->ID,
				'comment_date'	=>	current_time('mysql'),
				'comment_approved'	=>	10
			);
			$response_id = wp_insert_comment($data);
			if(!is_wp_error($response_id)){
				if( isset( $status ) )
					update_post_meta($post_id, 'wpo_ticket_status', $status);
					do_action('pressticket_new_response', $post_id, $response_content, NULL);
				return $this->send_response('success', __('Send the Responses successfully','wpo'));
			}
			return $this->send_response('error', __('ERROR: Please try again.','wpo') );
		}
		/** Handle Requests
		*	@return void 
		*/
		public function handle_request(){
			global $wp;
			$action = $wp->query_vars['action'];
			if(!$action)
				$this->send_response('error','API isn\'t defined.');
			return $this->action($action);
		}
		
		/** Response Handler
		*	This sends a JSON response to the browser
		*/
		protected function send_response($reps = 'error', $msg, $data = ''){
			$response['resp']	=	$reps;
			$response['message'] = $msg;
			if($data)
				$response['data'] = $data;
			header('content-type: application/json; charset=utf-8');
		    echo json_encode($response)."\n";
		    exit;
		}
	}
	new PT_API_Endpoint();	
}
